@@ -107,18 +107,10 @@ END
 sub Send($;$) {
     my ($id, $recipients) = (@_);
 
-    # This doesn't work if its not in a sub. Probably something to do with the
+    # This only works in a sub. Probably something to do with the
     # require abuse we do.
     GetVersionTable();
 
-    # Since any email recipients must be rederived if the user has not
-    # been rederived since the most recent group change, figure out when that
-    # is once and determine the need to rederive users using the same DB 
-    # access that gets the user's email address each time a person is 
-    # processed.
-    SendSQL("SELECT MAX(last_changed) FROM groups");
-    ($last_changed) = FetchSQLData();
-
     # Make sure to clean up _all_ package vars here. Yuck...
     $nametoexclude = $recipients->{'changer'} || "";
     @{$force{'CClist'}} = (exists $recipients->{'cc'} && 
@@ -518,10 +510,10 @@ sub getEmailAttributes (\%\@$) {
             push (@flags, 'CC');
         }
 
-        # These next few lines are for finding out who's been added
-        # to the Owner, QA, CC, etc. fields.  It does not effect
-        # the @flags array at all, but is run here because it does
-        # effect filtering later and we're already in the loop.
+        # These next few lines find out who has been added
+        # to the Owner, QA, CC, etc. fields.  They do not affect
+        # the @flags array at all, but are run here because they
+        # affect filtering later and we're already in the loop.
         if ($fieldName eq 'AssignedTo') {
             push (@{$force{'Owner'}}, $new);
         } elsif ($fieldName eq 'QAcontact') {
@@ -563,9 +555,9 @@ sub getEmailAttributes (\%\@$) {
 
 sub filterEmailGroup ($$$) {
     # This function figures out who should receive email about the bug
-    # based on the user's role with regard to the bug (assignee, reporter 
+    # based on the user's role with respect to the bug (assignee, reporter, 
     # etc.), the changes that occurred (new comments, attachment added, 
-    # status changed etc.) and the user's email preferences.
+    # status changed, etc.) and the user's email preferences.
     
     # Returns a filtered list of those users who would receive email
     # about these changes, and adds the names of those users who would
@@ -577,7 +569,7 @@ sub filterEmailGroup ($$$) {
     my @users = split( /,/ , $users );
     
     # Treat users who are in the process of being removed from this role
-    # as if they were still in it.
+    # as if they still have it.
     push @users, @{$force{$role}};
 
     # If this installation supports user watching, add to the list those
@@ -607,7 +599,7 @@ sub filterEmailGroup ($$$) {
         # (no idea why unregistered users should even be on this list,
         # but the code that was here before I re-wrote it allows this),
         # then we do not have any preferences for them, so assume the
-        # default preference to receive all mail for any reason.
+        # default preference is to receive all mail.
         my $userid = DBname_to_id($user);
         if (!$userid) {
             push(@recipients, $user);
@@ -618,10 +610,10 @@ sub filterEmailGroup ($$$) {
         SendSQL("SELECT emailflags FROM profiles WHERE userid = $userid");
         my $prefs = FetchOneColumn();
         
-        # If the user's preferences are empty, assume the default preference 
-        # to receive all mail.  This happens when the installation upgraded
-        # from a version of Bugzilla without email preferences to one with
-        # them, but the user has not set their preferences yet.
+        # If the user's preferences are empty, it means the user has not set
+        # their mail preferences after the installation upgraded from a
+        # version of Bugzilla without email preferences to one with them. In
+        # this case, assume they want to receive all mail.
         if (!defined($prefs) || $prefs !~ /email/) {
             push(@recipients, $user);
             next;
@@ -637,7 +629,7 @@ sub filterEmailGroup ($$$) {
         # If this user is the one who made the change in the first place,
         # and they prefer not to receive mail about their own changes,
         # filter them from the list.
-        if (lc($user) eq $nametoexclude && $prefs{'ExcludeSelf'} eq 'on') {
+        if (lc($user) eq lc($nametoexclude) && $prefs{'ExcludeSelf'} eq 'on') {
             push(@excludedAddresses, $user);
             next;
         }
@@ -710,19 +702,13 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
       return;
     }
 
-        
-    SendSQL("SELECT userid, (refreshed_when > " . SqlQuote($last_changed) . 
-            ") FROM profiles WHERE login_name = " . SqlQuote($person));
-    my ($userid, $current) = (FetchSQLData());
+    # This routine should really get passed a userid
+    # This rederives groups as a side effect
+    my $user = Bugzilla::User->new_from_login($person);
+    my $userid = $user->id;
 
     $seen{$person} = 1;
 
-    detaint_natural($userid);
-
-    if (!$current) {
-        DeriveGroup($userid);
-    }
-
     # if this person doesn't have permission to see info on this bug, 
     # return.
     #
@@ -732,15 +718,14 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
     # quietly disappear from their radar.
     #
     return unless CanSeeBug($id, $userid);
-    
 
     #  Drop any non-insiders if the comment is private
-     return if (Param("insidergroup") && 
+    return if (Param("insidergroup") && 
                ($anyprivate != 0) && 
-               (!UserInGroup(Param("insidergroup"), $userid)));
+               (!$user->groups->{Param("insidergroup")}));
 
     # We shouldn't send changedmail if this is a dependency mail, and any of 
-    # the depending bugs is not visible to the user.
+    # the depending bugs are not visible to the user.
     foreach my $dep_id (@depbugs) {
         my $save_id = $dep_id;
         detaint_natural($dep_id) || warn("Unexpected Error: \@depbugs contains a non-numeric value: '$save_id'")
@@ -759,10 +744,10 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
         if (! $value) {
           next;
         }
-        # Don't send estimated_time if user not in the group, or not enabled
+        # Only send estimated_time if it is enabled and the user is in the group
         if ($f ne 'estimated_time' ||
-            UserInGroup(Param('timetrackinggroup'), $userid)) { 
-             
+            $user->groups->{Param('timetrackinggroup')}) {
+
             my $desc = $fielddescription{$f};
             $head .= FormatDouble($desc, $value);
         }
@@ -781,7 +766,7 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
          ($diff->{'fieldname'} eq 'estimated_time' ||
          $diff->{'fieldname'} eq 'remaining_time' ||
          $diff->{'fieldname'} eq 'work_time')) {
-            if (UserInGroup(Param("timetrackinggroup"), $userid)) {
+            if ($user->groups->{Param("timetrackinggroup")}) {
                 $add_diff = 1;
             }
         } else {
@@ -846,7 +831,7 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
 # since if the bug didn't change, you wouldn't be getting mail
 # in the first place! see http://bugzilla.mozilla.org/show_bug.cgi?id=29820 
 # for details.
-    $substs{"neworchanged"} = $isnew ? 'New: ' : '';
+    $substs{"neworchanged"} = $isnew ? ' New: ' : '';
     $substs{"to"} = $person;
     $substs{"cc"} = '';
     $substs{"bugid"} = $id;
