@@ -34,6 +34,7 @@ use vars qw($userid);
 package Bugzilla::Search;
 
 use Bugzilla::Config;
+use Bugzilla::Error;
 use Bugzilla::Util;
 
 use Date::Format;
@@ -57,6 +58,7 @@ sub init {
     my $self = shift;
     my $fieldsref = $self->{'fields'};
     my $params = $self->{'params'};
+    my $user = $self->{'user'} || Bugzilla->user;
 
     my $debug = 0;
         
@@ -103,7 +105,7 @@ sub init {
         my $c = trim($params->param('votes'));
         if ($c ne "") {
             if ($c !~ /^[0-9]*$/) {
-                &::ThrowUserError("illegal_at_least_x_votes",
+                ThrowUserError("illegal_at_least_x_votes",
                                   { value => $c });
             }
             push(@specialchart, ["votes", "greaterthan", $c - 1]);
@@ -120,12 +122,23 @@ sub init {
 
     # If the user has selected all of either status or resolution, change to
     # selecting none. This is functionally equivalent, but quite a lot faster.
+    # Also, if the status is __open__ or __closed__, translate those
+    # into their equivalent lists of open and closed statuses.
     if ($params->param('bug_status')) {
         my @bug_statuses = $params->param('bug_status');
-        
-        if (scalar(@bug_statuses) == scalar(@::legal_bug_status)) {
+        if (scalar(@bug_statuses) == scalar(@::legal_bug_status) 
+            || $bug_statuses[0] eq "__all__")
+        {
             $params->delete('bug_status');
         }
+        elsif ($bug_statuses[0] eq '__open__') {
+            $params->param('bug_status', map(&::IsOpenedState($_) ? $_ : undef, 
+                                             @::legal_bug_status));
+        }
+        elsif ($bug_statuses[0] eq "__closed__") {
+            $params->param('bug_status', map(&::IsOpenedState($_) ? undef : $_, 
+                                             @::legal_bug_status));
+        }
     }
     
     if ($params->param('resolution')) {
@@ -212,64 +225,65 @@ sub init {
         }
     }
 
-
-    if (defined $params->param('changedin')) {
-        my $c = trim($params->param('changedin'));
-        if ($c ne "") {
-            if ($c !~ /^[0-9]*$/) {
-                &::ThrowUserError("illegal_changed_in_last_x_days",
-                                 { value => $c });
-            }
-            push(@specialchart, ["changedin",
-                                 "lessthan", $c + 1]);
+    # 2003-05-20: The 'changedin' field is no longer in the UI, but we continue
+    # to process it because it will appear in stored queries and bookmarks.
+    my $changedin = $params->param('changedin') || '';
+    $changedin = trim($changedin);
+    if ($changedin) {
+        if ($changedin !~ /^[0-9]*$/) {
+            ThrowUserError("illegal_changed_in_last_x_days",
+                              { value => $changedin });
         }
+        push(@specialchart, ["changedin", "lessthan", $changedin + 1]);
     }
 
+    my $chfieldfrom = $params->param('chfieldfrom') || '';
+    my $chfieldto = $params->param('chfieldto') || '';
     my @chfield = $params->param('chfield');
+    my $chvalue = $params->param('chfieldvalue') || '';
+    my $lcchfieldfrom = trim(lc($chfieldfrom));
+    my $lcchfieldto = trim(lc($chfieldto));
+    $chvalue = trim($chvalue);
 
-    if (@chfield) {
-        my $which = lsearch(\@chfield, "[Bug creation]");
-        if ($which >= 0) {
-            splice(@chfield, $which, 1);
-            push(@specialchart, ["creation_ts", "greaterthan",
-                                 SqlifyDate($params->param('chfieldfrom'))]);
-            my $to = $params->param('chfieldto');
-            if (defined $to) {
-                $to = trim($to);
-                if ($to ne "" && $to !~ /^now$/i) {
-                    push(@specialchart, ["creation_ts", "lessthan",
-                                         SqlifyDate($to)]);
+    $lcchfieldfrom = '' if ($lcchfieldfrom eq 'now');
+    $lcchfieldto = '' if ($lcchfieldto eq 'now');
+    if ($lcchfieldfrom ne '' || $lcchfieldto ne '') {
+        my $sql_chfrom = $lcchfieldfrom ? &::SqlQuote(SqlifyDate($lcchfieldfrom)):'';
+        my $sql_chto   = $lcchfieldto   ? &::SqlQuote(SqlifyDate($lcchfieldto))  :'';
+        my $sql_chvalue = $chvalue ne '' ? &::SqlQuote($chvalue) : '';
+        if(!@chfield) {
+            push(@wherepart, "bugs.delta_ts >= $sql_chfrom") if ($sql_chfrom);
+            push(@wherepart, "bugs.delta_ts <= $sql_chto") if ($sql_chto);
+        } else {
+            push(@supptables, "bugs_activity actcheck");
+            my $sql_bugschanged = '';
+            my @list;
+            foreach my $f (@chfield) {
+                if ($f eq "[Bug creation]") {
+                    my @l;
+                    push(@l, "creation_ts >= $sql_chfrom") if($sql_chfrom);
+                    push(@l, "creation_ts <= $sql_chto") if($sql_chto);
+                    $sql_bugschanged = "(" . join(' AND ', @l) . ")";
+                } else {
+                    push(@list, "\nactcheck.fieldid = " . &::GetFieldID($f));
                 }
             }
-        }
-    }
-
-    if (@chfield) {
-        push(@supptables, "bugs_activity actcheck");
-
-        my @list;
-        foreach my $f (@chfield) {
-            push(@list, "\nactcheck.fieldid = " . &::GetFieldID($f));
-        }
-        push(@wherepart, "actcheck.bug_id = bugs.bug_id");
-        push(@wherepart, "(" . join(' OR ', @list) . ")");
-        push(@wherepart, "actcheck.bug_when >= " .
-             &::SqlQuote(SqlifyDate($params->param('chfieldfrom'))));
-        my $to = $params->param('chfieldto');
-        if (defined $to) {
-            $to = trim($to);
-            if ($to ne "" && $to !~ /^now$/i) {
-                push(@wherepart, "actcheck.bug_when <= " .
-                     &::SqlQuote(SqlifyDate($to)));
-            }
-        }
-        my $value = $params->param('chfieldvalue');
-        if (defined $value) {
-            $value = trim($value);
-            if ($value ne "") {
-                push(@wherepart, "actcheck.added = " .
-                     &::SqlQuote($value))
+            if(@list) {
+                $sql_bugschanged .= ' OR ' if($sql_bugschanged ne '');
+                $sql_bugschanged .= "(actcheck.bug_id = bugs.bug_id AND " .
+                                       "(" . join(' OR ', @list) . ")";
+                if($sql_chfrom) {
+                    $sql_bugschanged .= " AND actcheck.bug_when >= $sql_chfrom";
+                }
+                if($sql_chto) {
+                    $sql_bugschanged .= " AND actcheck.bug_when <= $sql_chto";
+                }
+                if($sql_chvalue) {
+                    $sql_bugschanged .= " AND actcheck.added = $sql_chvalue";
+                }
+                $sql_bugschanged .= ')';
             }
+            push(@wherepart, "($sql_bugschanged)");
         }
     }
 
@@ -286,6 +300,10 @@ sub init {
         }
     }
 
+    if (defined $params->param('content')) {
+        push(@specialchart, ['content', 'matches', $params->param('content')]);
+    }
+
     my $chartid;
     my $sequence = 0;
     # $type_id is used by the code that queries for attachment flags.
@@ -363,6 +381,55 @@ sub init {
              push(@wherepart, "$table.bug_id = bugs.bug_id");
              $term = "$table.bug_when > " . &::SqlQuote(SqlifyDate($v));
          },
+         "^content,matches" => sub {
+             # "content" is an alias for columns containing text for which we
+             # can search a full-text index and retrieve results by relevance, 
+             # currently just bug comments (and summaries to some degree).
+             # There's only one way to search a full-text index
+             # ("MATCH (...) AGAINST (...)"), so we only accept the "matches"
+             # operator, which is specific to full-text index searches.
+
+             # Add the longdescs table to the query so we can search comments.
+             my $table = "longdescs_$chartid";
+             push(@supptables, "INNER JOIN longdescs $table ON bugs.bug_id " . 
+                               "= $table.bug_id");
+             if (Param("insidergroup") 
+                 && !&::UserInGroup(Param("insidergroup")))
+             {
+                 push(@wherepart, "$table.isprivate < 1");
+             }
+             push(@wherepart, "$table.bug_id = bugs.bug_id");
+
+             # Create search terms to add to the SELECT and WHERE clauses.
+             # $term1 searches comments.
+             # $term2 searches summaries, which contributes to the relevance
+             # ranking in SELECT but doesn't limit which bugs get retrieved.
+             my $term1 = "MATCH($table.thetext) AGAINST(".&::SqlQuote($v).")";
+             my $term2 = "MATCH(bugs.short_desc) AGAINST(".&::SqlQuote($v).")";
+
+             # The term to use in the WHERE clause.
+             $term = $term1;
+
+             # In order to sort by relevance, we SELECT the relevance value
+             # and give it an alias so we can add it to the SORT BY clause
+             # when we build that clause in buglist.cgi.  We also flag the
+             # query in Bugzilla with the "sorted_by_relevance" flag
+             # so buglist.cgi knows to sort by relevance instead of anything
+             # else the user selected.
+             #
+             # Note: MySQL calculates relevance for each comment separately,
+             # so we need to do some additional calculations to get an overall
+             # relevance value, which we do by calculating the average (mean)
+             # comment relevance and then adding the summary relevance, if any.
+             # This weights summary relevance heavily, which makes sense
+             # since summaries are short and thus highly significant.
+             #
+             # Note: We should be calculating the average relevance of all
+             # comments for a bug, not just matching comments, but that's hard
+             # (see http://bugzilla.mozilla.org/show_bug.cgi?id=145588#c35).
+             push(@fields, "(SUM($term1)/COUNT($term1) + $term2) AS relevance");
+             $self->{'sorted_by_relevance'} = 1;
+         },
          "^long_?desc," => sub {
              my $table = "longdescs_$chartid";
              push(@supptables, "longdescs $table");
@@ -460,10 +527,10 @@ sub init {
                  $t = "greaterthan";
              }
              if ($field eq "ispatch" && $v ne "0" && $v ne "1") {
-                 &::ThrowUserError("illegal_attachment_is_patch");
+                 ThrowUserError("illegal_attachment_is_patch");
              }
              if ($field eq "isobsolete" && $v ne "0" && $v ne "1") {
-                 &::ThrowUserError("illegal_is_obsolete");
+                 ThrowUserError("illegal_is_obsolete");
              }
              $f = "$table.$field";
          },
@@ -766,7 +833,7 @@ sub init {
 # by pressing the OR button at the right end of the chart. Extra
 # charts are created by pressing "Add another boolean chart".
 #
-# Each chart consists of an artibrary number of rows and columns.
+# Each chart consists of an arbitrary number of rows and columns.
 # The terms within a row are ORed together. The expressions represented
 # by each row are ANDed together. The expressions represented by each
 # chart are ANDed together.
@@ -793,7 +860,7 @@ sub init {
 # The terms within a single row of a boolean chart are all constraints
 # on a single piece of data.  If you're looking for a bug that has two
 # different people cc'd on it, then you need to use two boolean charts.
-# This will find bugs with one CC mathing 'foo@blah.org' and and another
+# This will find bugs with one CC matching 'foo@blah.org' and and another
 # CC matching 'bar@blah.org'.
 #
 # --------------------------------------------------------------
@@ -816,7 +883,7 @@ sub init {
 # bar@blah.org
 # --------------------------------------------------------------
 
-# $chartid is the number of the current chart whose SQL we're contructing
+# $chartid is the number of the current chart whose SQL we're constructing
 # $row is the current row of the current chart
 
 # names for table aliases are constructed using $chartid and $row
@@ -864,12 +931,12 @@ sub init {
                 # chart -1 is generated by other code above, not from the user-
                 # submitted form, so we'll blindly accept any values in chart -1
                 if ((!$chartfields{$f}) && ($chart != -1)) {
-                    &::ThrowCodeError("invalid_field_name", {field => $f});
+                    ThrowCodeError("invalid_field_name", {field => $f});
                 }
 
                 # This is either from the internal chart (in which case we
                 # already know about it), or it was in %chartfields, so it is
-                # a valid field name, which means that its ok.
+                # a valid field name, which means that it's ok.
                 trick_taint($f);
                 $q = &::SqlQuote($v);
                 my $func;
@@ -898,9 +965,10 @@ sub init {
                 }
                 else {
                     # This field and this type don't work together.
-                    $::vars->{'field'} = $params->param("field$chart-$row-$col");
-                    $::vars->{'type'} = $params->param("type$chart-$row-$col");
-                    &::ThrowCodeError("field_type_mismatch");
+                    ThrowCodeError("field_type_mismatch",
+                                   { field => $params->param("field$chart-$row-$col"),
+                                     type => $params->param("type$chart-$row-$col"),
+                                   });
                 }
             }
             if (@orlist) {
@@ -929,23 +997,24 @@ sub init {
                 " LEFT JOIN bug_group_map " .
                 " ON bug_group_map.bug_id = bugs.bug_id ";
 
-    if (defined @{$::vars->{user}{groupids}} && @{$::vars->{user}{groupids}} > 0) {
-        $query .= " AND bug_group_map.group_id NOT IN (" . join(',', @{$::vars->{user}{groupids}}) . ") ";
-    }
+    if ($user) {
+        if (%{$user->groups}) {
+            $query .= " AND bug_group_map.group_id NOT IN (" . join(',', values(%{$user->groups})) . ") ";
+        }
 
-    if ($::vars->{user}{userid}) {
-        $query .= " LEFT JOIN cc ON cc.bug_id = bugs.bug_id AND cc.who = $::userid ";
+        $query .= " LEFT JOIN cc ON cc.bug_id = bugs.bug_id AND cc.who = " . $user->id;
     }
 
     $query .= " WHERE " . join(' AND ', (@wherepart, @andlist)) .
               " AND ((bug_group_map.group_id IS NULL)";
 
-    if ($::vars->{user}{userid}) {
-        $query .= "    OR (bugs.reporter_accessible = 1 AND bugs.reporter = $::userid) " .
+    if ($user) {
+        my $userid = $user->id;
+        $query .= "    OR (bugs.reporter_accessible = 1 AND bugs.reporter = $userid) " .
               "    OR (bugs.cclist_accessible = 1 AND cc.who IS NOT NULL) " .
-              "    OR (bugs.assigned_to = $::userid) ";
+              "    OR (bugs.assigned_to = $userid) ";
         if (Param('useqacontact')) {
-            $query .= "OR (bugs.qa_contact = $::userid) ";
+            $query .= "OR (bugs.qa_contact = $userid) ";
         }
     }
 
@@ -969,6 +1038,10 @@ sub init {
 sub SqlifyDate {
     my ($str) = @_;
     $str = "" if !defined $str;
+    if ($str eq "") {
+        my ($sec, $min, $hour, $mday, $month, $year, $wday) = localtime(time());
+        return sprintf("%4d-%02d-%02d 00:00:00", $year+1900, $month+1, $mday);
+    }
     if ($str =~ /^-?(\d+)([dDwWmMyY])$/) {   # relative date
         my ($amount, $unit, $date) = ($1, lc $2, time);
         my ($sec, $min, $hour, $mday, $month, $year, $wday)  = localtime($date);
@@ -992,13 +1065,13 @@ sub SqlifyDate {
     }
     my $date = str2time($str);
     if (!defined($date)) {
-        &::ThrowUserError("illegal_date", { date => $str });
+        ThrowUserError("illegal_date", { date => $str });
     }
     return time2str("%Y-%m-%d %H:%M:%S", $date);
 }
 
-# ListIDsForEmail returns an string with a comma-joined list
-# of userids matching an email address
+# ListIDsForEmail returns a string with a comma-joined list
+# of userids matching email addresses
 # according to the type specified.
 # Currently, this only supports anyexact and substring matches.
 # Substring matches will return up to 50 matching userids
