@@ -42,7 +42,7 @@ use Bugzilla::Error;
 
 sub CGI_pl_sillyness {
     my $zz;
-    $zz = %::dontchange;
+    $zz = $::buffer;
 }
 
 use CGI::Carp qw(fatalsToBrowser);
@@ -55,11 +55,11 @@ use vars qw($template $vars);
 # to the user about the downtime.  (do)editparams.cgi is exempted from
 # this message, of course, since it needs to be available in order for
 # the administrator to open Bugzilla back up.
-if (Param("shutdownhtml") && $0 !~ m:[\\/](do)?editparams.cgi$:) {
+if (Param("shutdownhtml") && $0 !~ m:(^|[\\/])(do)?editparams\.cgi$:) {
     $::vars->{'message'} = "shutdown";
     
     # Return the appropriate HTTP response headers.
-    print "Content-Type: text/html\n\n";
+    print Bugzilla->cgi->header();
     
     # Generate and return an HTML message about the downtime.
     $::template->process("global/message.html.tmpl", $::vars)
@@ -95,14 +95,15 @@ sub CheckFormField (\%$;\@) {
 
         SendSQL("SELECT description FROM fielddefs WHERE name=" . SqlQuote($fieldname));
         my $result = FetchOneColumn();
+        my $field;
         if ($result) {
-            $vars->{'field'} = $result;
+            $field = $result;
         }
         else {
-            $vars->{'field'} = $fieldname;
+            $field = $fieldname;
         }
         
-        ThrowCodeError("illegal_field", undef, "abort");
+        ThrowCodeError("illegal_field", { field => $field }, "abort");
       }
 }
 
@@ -113,8 +114,7 @@ sub CheckFormFieldDefined (\%$) {
        ) = @_;
 
     if (!defined $formRef->{$fieldname}) {
-        $vars->{'field'} = $fieldname;  
-        ThrowCodeError("undefined_field");
+        ThrowCodeError("undefined_field", { field => $fieldname });
     }
 }
 
@@ -202,53 +202,6 @@ sub quietly_check_login {
     return Bugzilla->login($_[0] ? LOGIN_OPTIONAL : LOGIN_NORMAL);
 }
 
-# Populate a hash with information about this user. 
-sub GetUserInfo {
-    my ($userid) = (@_);
-    my %user;
-    my @queries;
-    my %groups;
-    my @groupids;
-    
-    # No info if not logged in
-    return \%user if ($userid == 0);
-    
-    $user{'login'} = $::COOKIE{"Bugzilla_login"};
-    $user{'userid'} = $userid;
-    
-    SendSQL("SELECT mybugslink, realname " . 
-            "FROM profiles WHERE userid = $userid");
-    ($user{'showmybugslink'}, $user{'realname'}) = FetchSQLData();
-
-    SendSQL("SELECT name, query, linkinfooter FROM namedqueries " .
-            "WHERE userid = $userid");
-    while (MoreSQLData()) {
-        my %query;
-        ($query{'name'}, $query{'query'}, $query{'linkinfooter'}) = 
-                                                                 FetchSQLData();
-        push(@queries, \%query);    
-    }
-
-    $user{'queries'} = \@queries;
-
-    $user{'canblessany'} = UserCanBlessAnything();
-
-    SendSQL("SELECT DISTINCT id, name FROM groups, user_group_map " .
-            "WHERE groups.id = user_group_map.group_id " .
-            "AND user_id = $userid " .
-            "AND NOT isbless");
-    while (MoreSQLData()) {
-        my ($id, $name) = FetchSQLData();    
-        push(@groupids,$id);
-        $groups{$name} = 1;
-    }
-
-    $user{'groups'} = \%groups;
-    $user{'groupids'} = \@groupids;
-
-    return \%user;
-}
-
 sub CheckEmailSyntax {
     my ($addr) = (@_);
     my $match = Param('emailregexp');
@@ -288,81 +241,6 @@ sub PutFooter {
       || ThrowTemplateError($::template->error());
 }
 
-###############################################################################
-# Error handling
-#
-# If you are doing incremental output, set $vars->{'header_done'} once you've
-# done the header.
-#
-# You can call Throw*Error with extra template variables in one pass by using
-# the $extra_vars hash reference parameter:
-# ThrowUserError("some_tag", { bug_id => $bug_id, size => 127 });
-###############################################################################
-
-# For "this shouldn't happen"-type places in the code.
-# The contents of $extra_vars get printed out in the template - useful for
-# debugging info.
-sub ThrowCodeError {
-  ($vars->{'error'}, my $extra_vars, my $unlock_tables) = (@_);
-
-  SendSQL("UNLOCK TABLES") if $unlock_tables;
-
-  # If we don't have this test here, then the %@extra_vars vivifies
-  # the hashref, and then setting $vars->{'variables'} uses an empty hashref
-  # so the error template prints out a bogus header for the empty hash
-  if (defined $extra_vars) {
-      # Copy the extra_vars into the vars hash 
-      foreach my $var (keys %$extra_vars) {
-          $vars->{$var} = $extra_vars->{$var};
-      }
-
-      # We may one day log something to file here also.
-      $vars->{'variables'} = $extra_vars;
-  }
-  
-  print "Content-type: text/html\n\n" if !$vars->{'header_done'};
-  $template->process("global/code-error.html.tmpl", $vars)
-    || ThrowTemplateError($template->error());
-    
-  exit;
-}
-
-# This function should only be called if a template->process() fails.
-# It tries another template first, because often one template being
-# broken or missing doesn't mean that they all are. But it falls back on
-# a print statement.
-# The Content-Type will already have been printed.
-sub ThrowTemplateError {
-    ($vars->{'template_error_msg'}) = (@_);
-    $vars->{'error'} = "template_error";
-    
-    # Try a template first; but if this one fails too, fall back
-    # on plain old print statements.
-    if (!$template->process("global/code-error.html.tmpl", $vars)) {
-        my $maintainer = Param('maintainer');
-        my $error = html_quote($vars->{'template_error_msg'});
-        my $error2 = html_quote($template->error());
-        print <<END;
-        <tt>
-          <p>
-            Bugzilla has suffered an internal error. Please save this page and 
-            send it to $maintainer with details of what you were doing at the 
-            time this message appeared.
-          </p>
-          <script type="text/javascript"> <!--
-            document.write("<p>URL: " + document.location + "</p>");
-          // -->
-          </script>
-          <p>Template->process() failed twice.<br>
-          First error: $error<br>
-          Second error: $error2</p>
-        </tt>
-END
-    }
-    
-    exit;  
-}
-
 sub CheckIfVotedConfirmed {
     my ($id, $who) = (@_);
     SendSQL("SELECT bugs.votes, bugs.bug_status, products.votestoconfirm, " .
