@@ -47,6 +47,7 @@
 
 use strict;
 use vars qw(@param_list);
+use File::Spec; # for find_languages
 
 # Checking functions for the various values
 # Some generic checking functions are included in Bugzilla::Config
@@ -148,6 +149,35 @@ sub check_loginmethod {
     return "";
 }
 
+sub check_languages {
+    my @languages = split /,/, trim($_);
+    if(!scalar(@languages)) {
+       return "You need to specify a language tag."
+    }
+    foreach my $language (@languages) {
+       if(   ! -d 'template/'.trim($language).'/custom' 
+          && ! -d 'template/'.trim($language).'/default') {
+          return "The template directory for $language does not exist";
+       }
+    }
+    return "";
+}
+
+sub find_languages {
+    my @languages = ();
+    opendir(DIR, "template") || return "Can't open 'template' directory: $!";
+    my @langdirs = grep { /^[a-z-]+$/i } readdir(DIR);
+    closedir DIR;
+
+    foreach my $lang (@langdirs) {
+        next if($lang =~ /^CVS$/i);
+        my $deft_path = File::Spec->catdir('template', $lang, 'default');
+        my $cust_path = File::Spec->catdir('template', $lang, 'custom');
+        push(@languages, $lang) if(-d $deft_path or -d $cust_path);
+    }
+    return join(', ', @languages);
+}
+
 # OK, here are the parameter definitions themselves.
 #
 # Each definition is a hash with keys:
@@ -233,9 +263,11 @@ sub check_loginmethod {
            'to be displayed. Note that you must install the appropriate ' .
            'language pack before adding a language to this Param. The ' .
            'language used is the one in this list with the highest ' .
-           'q-value in the user\'s Accept-Language header.' ,
+           'q-value in the user\'s Accept-Language header.<br>' .
+           'Available languages: ' . find_languages() ,
    type => 't' ,
-   default => 'en'
+   default => 'en' ,
+   checker => \&check_languages
   },
 
   {
@@ -243,7 +275,8 @@ sub check_loginmethod {
    desc => 'The UI language Bugzilla falls back on if no suitable ' .
            'language is found in the user\'s Accept-Language header.' ,
    type => 't' ,
-   default => 'en'
+   default => 'en' ,
+   checker => \&check_languages
   },
 
   {
@@ -388,6 +421,14 @@ sub check_loginmethod {
    default => 'mail'
   },
 
+  {
+   name => 'LDAPfilter',
+   desc => 'LDAP filter to AND with the <tt>LDAPuidattribute</tt> for ' .
+           'filtering the list of valid users.',
+   type => 't',
+   default => '',
+  },
+
   {
    name => 'loginmethod',
    desc => 'The type of login authentication to use:
@@ -598,24 +639,6 @@ You will get this message once a day until you\'ve dealt with these bugs!
    default => 0
   },
 
-  {
-   name => 'nummilestones',
-   desc => 'If using Target Milestone, how many milestones do you wish to
-   appear?',
-   type => 't',
-   default => 10,
-   checker => \&check_numeric
-  },
-
-  {
-   name => 'curmilestone',
-   desc => 'If using Target Milestone, Which milestone are we working ' .
-           'toward right now?',
-   type => 't',
-   default => 1,
-   checker => \&check_numeric
-  },
-
   {
    name => 'musthavemilestoneonaccept',
    desc => 'If you are using Target Milestone, do you want to require that ' .
@@ -1049,6 +1072,73 @@ Reason: %reason%
    default => 1,
   },
 
+# Added for Patch Viewer stuff (attachment.cgi?action=diff)
+  {
+   name    => 'cvsroot',
+   desc    => 'The <a href="http://www.cvshome.org">CVS</a> root that most ' .
+              'users of your system will be using for "cvs diff".  Used in ' .
+              'Patch Viewer ("Diff" option on patches) to figure out where ' .
+              'patches are rooted even if users did the "cvs diff" from ' .
+              'different places in the directory structure.  (NOTE: if your ' .
+              'CVS repository is remote and requires a password, you must ' .
+              'either ensure the Bugzilla user has done a "cvs login" or ' .
+              'specify the password ' .
+              '<a href="http://www.cvshome.org/docs/manual/cvs_2.html#SEC26">as ' .
+              'part of the CVS root.</a>)  Leave this blank if you have no ' .
+              'CVS repository.',
+   type    => 't',
+   default => '',
+  },
+
+  {
+   name    => 'cvsroot_get',
+   desc    => 'The CVS root Bugzilla will be using to get patches from.  ' .
+              'Some installations may want to mirror their CVS repository on ' .
+              'the Bugzilla server or even have it on that same server, and ' .
+              'thus the repository can be the local file system (and much ' .
+              'faster).  Make this the same as cvsroot if you don\'t ' .
+              'understand what this is (if cvsroot is blank, make this blank ' .
+              'too).',
+   type    => 't',
+   default => '',
+  },
+
+  {
+   name    => 'bonsai_url',
+   desc    => 'The URL to a ' .
+              '<a href="http://www.mozilla.org/bonsai.html">Bonsai</a> ' .
+              'server containing information about your CVS repository.  ' .
+              'Patch Viewer will use this information to create links to ' .
+              'bonsai\'s blame for each section of a patch (it will append ' .
+              '"/cvsblame.cgi?..." to this url).  Leave this blank if you ' .
+              'don\'t understand what this is.',
+   type    => 't',
+   default => ''
+  },
+
+  {
+   name    => 'lxr_url',
+   desc    => 'The URL to an ' .
+              '<a href="http://sourceforge.net/projects/lxr">LXR</a> server ' .
+              'that indexes your CVS repository.  Patch Viewer will use this ' .
+              'information to create links to LXR for each file in a patch.  ' .
+              'Leave this blank if you don\'t understand what this is.',
+   type    => 't',
+   default => ''
+  },
+
+  {
+   name    => 'lxr_root',
+   desc    => 'Some LXR installations do not index the CVS repository from ' .
+              'the root--' .
+              '<a href="http://lxr.mozilla.org/mozilla">Mozilla\'s</a>, for ' .
+              'example, starts indexing under <code>mozilla/</code>.  This ' .
+              'means URLs are relative to that extra path under the root.  ' .
+              'Enter this if you have a similar situation.  Leave it blank ' .
+              'if you don\'t know what this is.',
+   type    => 't',
+   default => '',
+  },
 );
 
 1;
