@@ -39,8 +39,6 @@ use Bugzilla::Config qw(:DEFAULT ChmodDataFile);
 
 sub globals_pl_sillyness {
     my $zz;
-    $zz = @main::SqlStateStack;
-    $zz = $main::contenttypes;
     $zz = @main::default_column_list;
     $zz = $main::defaultqueryname;
     $zz = @main::enterable_products;
@@ -76,7 +74,6 @@ use DBI;
 
 use Date::Format;               # For time2str().
 use Date::Parse;               # For str2time().
-#use Carp;                       # for confess
 use RelationSet;
 
 # Use standard Perl libraries for cross-platform file/directory manipulation.
@@ -99,7 +96,12 @@ $::SIG{PIPE} = 'IGNORE';
 $::defaultqueryname = "(Default query)"; # This string not exposed in UI
 $::unconfirmedstate = "UNCONFIRMED";
 
+# The following subroutine is for debugging purposes only.
+# Uncommenting this sub and the $::SIG{__DIE__} trap underneath it will
+# cause any fatal errors to result in a call stack trace to help track
+# down weird errors.
 #sub die_with_dignity {
+#    use Carp;  # for confess()
 #    my ($err_msg) = @_;
 #    print $err_msg;
 #    confess($err_msg);
@@ -348,7 +350,7 @@ $::VersionTableLoaded = 0;
 sub GetVersionTable {
     return if $::VersionTableLoaded;
     my $mtime = ModTime("data/versioncache");
-    if (!defined $mtime || $mtime eq "") {
+    if (!defined $mtime || $mtime eq "" || !-r "data/versioncache") {
         $mtime = 0;
     }
     if (time() - $mtime > 3600) {
@@ -443,8 +445,10 @@ sub GenerateRandomPassword {
     $size ||= 8;
 
     # The list of characters that can appear in a randomly generated password.
-    # Note that users can put any character into a password they choose themselves.
-    my @pwchars = (0..9, 'A'..'Z', 'a'..'z', '-', '_', '!', '@', '#', '$', '%', '^', '&', '*');
+    # Note that users can put any character into a password they choose
+    # themselves.
+    my @pwchars = (0..9, 'A'..'Z', 'a'..'z', '-', '_', '!', '@', '#', '$',
+        '%', '^', '*');
 
     # The number of characters in the list.
     my $pwcharslen = scalar(@pwchars);
@@ -511,10 +515,9 @@ sub CanEditProductId {
     my $query = "SELECT group_id FROM group_control_map " .
                 "WHERE product_id = $productid " .
                 "AND canedit != 0 "; 
-    if ((defined @{$::vars->{user}{groupids}}) 
-        && (@{$::vars->{user}{groupids}} > 0)) {
+    if (defined Bugzilla->user && %{Bugzilla->user->groups}) {
         $query .= "AND group_id NOT IN(" . 
-                   join(',',@{$::vars->{user}{groupids}}) . ") ";
+                   join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
     $query .= "LIMIT 1";
     PushGlobalSQLState();
@@ -534,10 +537,9 @@ sub CanEnterProduct {
                 "LEFT JOIN group_control_map " .
                 "ON group_control_map.product_id = products.id " .
                 "AND group_control_map.entry != 0 ";
-    if ((defined @{$::vars->{user}{groupids}}) 
-        && (@{$::vars->{user}{groupids}} > 0)) {
+    if (defined Bugzilla->user && %{Bugzilla->user->groups}) {
         $query .= "AND group_id NOT IN(" . 
-                   join(',',@{$::vars->{user}{groupids}}) . ") ";
+                   join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
     $query .= "WHERE products.name = " . SqlQuote($productname) . " LIMIT 1";
     PushGlobalSQLState();
@@ -567,10 +569,9 @@ sub GetSelectableProducts {
         $query .= "AND group_control_map.membercontrol = " .
                   CONTROLMAPMANDATORY . " ";
     }
-    if ((defined @{$::vars->{user}{groupids}}) 
-        && (@{$::vars->{user}{groupids}} > 0)) {
+    if (defined Bugzilla->user && %{Bugzilla->user->groups}) {
         $query .= "AND group_id NOT IN(" . 
-                   join(',',@{$::vars->{user}{groupids}}) . ") ";
+                   join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
     $query .= "WHERE group_id IS NULL ORDER BY name";
     PushGlobalSQLState();
@@ -631,6 +632,26 @@ sub GetSelectableProductHash {
 }
 
 
+sub GetFieldDefs {
+    my $extra = "";
+    if (!UserInGroup(Param('timetrackinggroup'))) {
+        $extra = "WHERE name NOT IN ('estimated time', 'remaining_time', " .
+                 "'work_time', 'percentage_complete')";
+    }
+
+    my @fields;
+    PushGlobalSQLState();
+    SendSQL("SELECT name, description FROM fielddefs $extra ORDER BY sortkey");
+    while (MoreSQLData()) {
+        my ($name, $description) = FetchSQLData();
+        push(@fields, { name => $name, description => $description });
+    }
+    PopGlobalSQLState();
+
+    return(@fields);
+}
+
+
 sub CanSeeBug {
 
     my ($id, $userid) = @_;
@@ -723,99 +744,6 @@ sub Crypt {
     return $cryptedpassword;
 }
 
-# ConfirmGroup(userid) is called prior to any activity that relies
-# on user_group_map to ensure that derived group permissions are up-to-date.
-# Permissions must be rederived if ANY groups have a last_changed newer
-# than the profiles.refreshed_when value.
-sub ConfirmGroup {
-    my ($user, $locked) = (@_);
-    PushGlobalSQLState();
-    SendSQL("SELECT userid FROM profiles, groups WHERE userid = $user " .
-            "AND profiles.refreshed_when <= groups.last_changed ");
-    my $ret = FetchSQLData();
-    PopGlobalSQLState();
-    if ($ret) {
-        DeriveGroup($user, $locked);
-    }
-}
-
-# DeriveGroup removes and rederives all derived group permissions for
-# the specified user.  If $locked is true, Bugzilla has already locked
-# the necessary tables as part of a larger transaction, so this function
-# shouldn't lock them again (since then tables not part of this function's
-# lock will get unlocked).
-sub DeriveGroup {
-    my ($user, $locked) = (@_);
-    PushGlobalSQLState();
-
-    SendSQL("LOCK TABLES profiles WRITE, user_group_map WRITE, group_group_map READ, groups READ")
-      unless $locked;
-
-    # avoid races,  we are only as up to date as the BEGINNING of this process
-    SendSQL("SELECT login_name, NOW() FROM profiles WHERE userid = $user");
-    my ($login, $starttime) = FetchSQLData();
-    
-    # first remove any old derived stuff for this user
-    SendSQL("DELETE FROM user_group_map WHERE user_id = $user " .
-            "AND isderived = 1");
-
-    my %groupidsadded = ();
-    # add derived records for any matching regexps
-    SendSQL("SELECT id, userregexp FROM groups WHERE userregexp != ''");
-    while (MoreSQLData()) {
-        my ($groupid, $rexp) = FetchSQLData();
-        if ($login =~ m/$rexp/i) {        
-            PushGlobalSQLState();
-            $groupidsadded{$groupid} = 1;
-            SendSQL("INSERT INTO user_group_map " .
-                    "(user_id, group_id, isbless, isderived) " .
-                    "VALUES ($user, $groupid, 0, 1)");
-            PopGlobalSQLState();
-
-        }
-    }
-
-    # Get a list of the groups of which the user is a member.
-    my %groupidschecked = ();
-    my @groupidstocheck = ();
-    SendSQL("SELECT group_id FROM user_group_map WHERE user_id = $user
-             AND NOT isbless");
-    while (MoreSQLData()) {
-        my ($groupid) = FetchSQLData();
-        push(@groupidstocheck,$groupid);
-    }
-
-    # Each group needs to be checked for inherited memberships once.
-    while (@groupidstocheck) {
-        my $group = shift @groupidstocheck;
-        if (!defined($groupidschecked{"$group"})) {
-            $groupidschecked{"$group"} = 1;
-            SendSQL("SELECT grantor_id FROM group_group_map WHERE"
-                 . " member_id = $group AND NOT isbless");
-            while (MoreSQLData()) {
-                my ($groupid) = FetchSQLData();
-                if (!defined($groupidschecked{"$groupid"})) {
-                    push(@groupidstocheck,$groupid);
-                }
-                if (!$groupidsadded{$groupid}) {
-                    $groupidsadded{$groupid} = 1;
-                    PushGlobalSQLState();
-                    SendSQL("INSERT INTO user_group_map"
-                         . " (user_id, group_id, isbless, isderived)"
-                         . " VALUES ($user, $groupid, 0, 1)");
-                    PopGlobalSQLState();
-                }
-            }
-        }
-    }
-            
-    SendSQL("UPDATE profiles SET refreshed_when = " .
-            SqlQuote($starttime) . "WHERE userid = $user");
-    SendSQL("UNLOCK TABLES");
-    PopGlobalSQLState();
-};
-
-
 sub DBID_to_real_or_loginname {
     my ($id) = (@_);
     PushGlobalSQLState();
@@ -960,7 +888,7 @@ sub quoteUrls {
     my $tmp;
 
     # non-mailto protocols
-    my $protocol_re = qr/(afs|cid|ftp|gopher|http|https|mid|news|nntp|prospero|telnet|wais)/i;
+    my $protocol_re = qr/(afs|cid|ftp|gopher|http|https|mid|news|nntp|prospero|telnet|view-source|wais)/i;
 
     $text =~ s~\b(${protocol_re}:  # The protocol:
                   [^\s<>\"]+       # Any non-whitespace
@@ -982,10 +910,12 @@ sub quoteUrls {
 
     # attachment links - handle both cases separatly for simplicity
     $text =~ s~((?:^Created\ an\ |\b)attachment\s*\(id=(\d+)\))
-              ~<a href=\"attachment.cgi?id=$2&amp;action=view\">$1</a>~igx;
+              ~GetAttachmentLink($2, $1)
+              ~egmx;
 
     $text =~ s~\b(attachment\s*\#?\s*(\d+))
-              ~<a href=\"attachment.cgi?id=$2&amp;action=view\">$1</a>~igx;
+              ~GetAttachmentLink($2, $1)
+              ~egmx;
 
     # This handles bug a, comment b type stuff. Because we're using /g
     # we have to do this in one pattern, and so this is semi-messy.
@@ -1014,6 +944,60 @@ sub quoteUrls {
     return $text;
 }
 
+# GetAttachmentLink creates a link to an attachment,
+# including its title.
+
+sub GetAttachmentLink {
+    my ($attachid, $link_text) = @_;
+    detaint_natural($attachid) ||
+        die "GetAttachmentLink() called with non-integer attachment number";
+
+    # If we've run GetAttachmentLink() for this attachment before,
+    # %::attachlink will contain an anonymous array ref of relevant
+    # values.  If not, we need to get the information from the database.
+    if (! defined $::attachlink{$attachid}) {
+        # Make sure any unfetched data from a currently running query
+        # is saved off rather than overwritten
+        PushGlobalSQLState();
+
+        SendSQL("SELECT bug_id, isobsolete, description 
+                 FROM attachments WHERE attach_id = $attachid");
+
+        if (MoreSQLData()) {
+            my ($bugid, $isobsolete, $desc) = FetchSQLData();
+            my $title = "";
+            my $className = "";
+            if (CanSeeBug($bugid, $::userid)) {
+                $title = $desc;
+            }
+            if ($isobsolete) {
+                $className = "bz_obsolete";
+            }
+            $::attachlink{$attachid} = [value_quote($title), $className];
+        }
+        else {
+            # Even if there's nothing in the database, we want to save a blank
+            # anonymous array in the %::attachlink hash so the query doesn't get
+            # run again next time we're called for this attachment number.
+            $::attachlink{$attachid} = [];
+        }
+        # All done with this sidetrip
+        PopGlobalSQLState();
+    }
+
+    # Now that we know we've got all the information we're gonna get, let's
+    # return the link (which is the whole reason we were called :)
+    my ($title, $className) = @{$::attachlink{$attachid}};
+    # $title will be undefined if the bug didn't exist in the database.
+    if (defined $title) {
+        my $linkval = "attachment.cgi?id=$attachid&amp;action=view";
+        return qq{<a href="$linkval" class="$className" title="$title">$link_text</a>};
+    }
+    else {
+        return qq{$link_text};
+    }
+}
+
 # GetBugLink creates a link to a bug, including its title.
 # It takes either two or three parameters:
 #  - The bug number
@@ -1089,7 +1073,7 @@ sub GetLongDescriptionAsText {
     my $result = "";
     my $count = 0;
     my $anyprivate = 0;
-    my ($query) = ("SELECT profiles.login_name, DATE_FORMAT(longdescs.bug_when,'%Y.%d.%m %H:%i'), " .
+    my ($query) = ("SELECT profiles.login_name, DATE_FORMAT(longdescs.bug_when,'%Y.%m.%d %H:%i'), " .
                    "       longdescs.thetext, longdescs.isprivate " .
                    "FROM   longdescs, profiles " .
                    "WHERE  profiles.userid = longdescs.who " .
@@ -1190,23 +1174,8 @@ sub SplitEnumType {
     return @result;
 }
 
-# UserInGroup returns information aboout the current user if no second 
-# parameter is specified
 sub UserInGroup {
-    my ($groupname, $userid) = (@_);
-    if (!$userid) {
-        return $::vars->{'user'}{'groups'}{$_[0]};
-    }
-    PushGlobalSQLState();
-    $userid ||= $::userid;
-    SendSQL("SELECT groups.id FROM groups, user_group_map 
-        WHERE groups.id = user_group_map.group_id 
-        AND user_group_map.user_id = $userid
-        AND isbless = 0
-        AND groups.name = " . SqlQuote($groupname));
-    my $result = FetchOneColumn();
-    PopGlobalSQLState();
-    return defined($result);
+    return defined Bugzilla->user && defined Bugzilla->user->groups->{$_[0]};
 }
 
 sub UserCanBlessGroup {
@@ -1239,32 +1208,6 @@ sub UserCanBlessGroup {
     return $result; 
 }
 
-sub UserCanBlessAnything {
-    PushGlobalSQLState();
-    # check if user explicitly can bless a group
-    SendSQL("SELECT group_id FROM user_group_map 
-        WHERE user_id = $::userid AND isbless = 1");
-    my $result = FetchOneColumn();
-    PopGlobalSQLState();
-    if ($result) {
-        return 1;
-    }
-    PushGlobalSQLState();
-    # check if user is a member of a group that can bless this group
-    SendSQL("SELECT groups.id FROM groups, user_group_map, 
-        group_group_map 
-        WHERE groups.id = grantor_id 
-        AND user_group_map.user_id = $::userid
-        AND group_group_map.isbless = 1
-        AND user_group_map.group_id = member_id");
-    $result = FetchOneColumn();
-    PopGlobalSQLState();
-    if ($result) {
-        return 1;
-    }
-    return 0;
-}
-
 sub BugInGroup {
     my ($bugid, $groupname) = (@_);
     PushGlobalSQLState();
@@ -1355,7 +1298,7 @@ sub RemoveVotes {
     if ($who) {
         $whopart = " AND votes.who = $who";
     }
-    SendSQL("SELECT profiles.login_name, profiles.userid, votes.count, " .
+    SendSQL("SELECT profiles.login_name, profiles.userid, votes.vote_count, " .
             "products.votesperuser, products.maxvotesperbug " .
             "FROM profiles " . 
             "LEFT JOIN votes ON profiles.userid = votes.who " .
@@ -1393,7 +1336,7 @@ sub RemoveVotes {
 
             my $newvotestext;
             if ($newvotes) {
-                SendSQL("UPDATE votes SET count = $newvotes " .
+                SendSQL("UPDATE votes SET vote_count = $newvotes " .
                         "WHERE bug_id = $id AND who = $userid");
                 $s = $newvotes == 1 ? "" : "s";
                 $newvotestext = "You still have $newvotes vote$s on this bug."
@@ -1436,7 +1379,7 @@ sub RemoveVotes {
                 close SENDMAIL;
             }
         }
-        SendSQL("SELECT SUM(count) FROM votes WHERE bug_id = $id");
+        SendSQL("SELECT SUM(vote_count) FROM votes WHERE bug_id = $id");
         my $v = FetchOneColumn();
         $v ||= 0;
         SendSQL("UPDATE bugs SET votes = $v, delta_ts = delta_ts " .
@@ -1536,7 +1479,7 @@ sub GetFormat {
     {
         'template'    => $template ,
         'extension'   => $ctype ,
-        'ctype' => $::contenttypes->{$ctype} ,
+        'ctype'       => Bugzilla::Constants::contenttypes->{$ctype} ,
     };
 }
 
