@@ -1,4 +1,4 @@
-#!/usr/bonsaitools/bin/perl -w
+#!/usr/bin/perl -w
 # -*- Mode: perl; indent-tabs-mode: nil -*-
 #
 # The contents of this file are subject to the Mozilla Public
@@ -27,13 +27,16 @@ use strict;
 use Test::Harness qw(&runtests $verbose);
 
 $verbose = 0;
+my $onlytest = "";
 
 foreach (@ARGV) {
     if (/^(?:-v|--verbose)$/) {
         $verbose = 1;
-        last;
+    }
+    else {
+        $onlytest = $_;
     }
 }
 
-runtests(glob("t/*.t"));
+runtests(glob("t/$onlytest*.t"));
 
